@@ -51,7 +51,7 @@ static void add_morpheme_unmarked(Sentence sent, char *join_buff,
                                   const char *wm, Morpheme_type mt)
 {
 	const char infix_mark = INFIX_MARK(sent->dict->affix_table);
-	const char *sm =  strrchr(wm, SUBSCRIPT_MARK);
+	const char *sm =  get_word_subscript(wm);
 
 	if (NULL == sm) sm = (char *)wm + strlen(wm);
 
@@ -240,9 +240,13 @@ void remove_empty_words(Linkage lkg)
 			continue;
 		}
 
+#if USE_SAT_SOLVER
 		Disjunct *cdtmp = cdj[j];
+#endif
 		cdj[j] = cdj[i];
+#if USE_SAT_SOLVER
 		cdj[i] = cdtmp; /* The SAT parser frees chosen_disjuncts elements. */
+#endif
 		remap[i] = j;
 		j++;
 		wgp++;
@@ -258,7 +262,7 @@ void remove_empty_words(Linkage lkg)
 
 /**
  * This takes the Wordgraph path array and uses it to
- * compute the chosen_words array.  "I.xx" suffixes are eliminated.
+ * compute the chosen_words array.  "I" suffixes are eliminated.
  *
  * Input:
  *
@@ -461,9 +465,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 		else
 		{
 			/* This word has a linkage. */
-
 			/* TODO: Suppress "virtual-morphemes", currently the dictcap ones. */
-			char *sm;
 
 			t = cdj->word_string;
 			/* Print the subscript, as in "dog.n" as opposed to "dog". */
@@ -474,11 +476,11 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 			}
 			else
 			{
-				/* Get rid of those ugly ".Ixx" */
+				/* Get rid of those ugly ".I" */
 				if (is_idiom_word(t))
 				{
 					s = strdupa(t);
-					sm = strrchr(s, SUBSCRIPT_MARK); /* Possible double subscript. */
+					char *sm = (char *)get_word_subscript(s);
 					UNREACHABLE(NULL == sm); /* We know it has a subscript. */
 					*sm = '\0';
 					t = string_set_add(s, sent->string_set);
@@ -566,7 +568,8 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 								}
 							}
 
-							sm =  strchr(cdjp[i+m]->word_string, SUBSCRIPT_MARK);
+							const char *sm =
+								get_word_subscript(cdjp[i+m]->word_string);
 
 							if (NULL != sm)
 							{
@@ -629,7 +632,7 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 			{
 
 				s = strdupa(t);
-				sm = strrchr(s, SUBSCRIPT_MARK);
+				char *sm = get_word_subscript(s);
 				if (sm) *sm = SUBSCRIPT_DOT;
 
 				if ((!(w->status & WS_GUESS) && (w->status & WS_INDICT))
@@ -718,9 +721,13 @@ static void compute_chosen_words(Sentence sent, Linkage linkage,
 			linkage->word[j] = chosen_words[i];
 			//chosen_words[i] = cwtmp;
 
+#if USE_SAT_SOLVER
 			Disjunct *cdtmp = cdj[j];
+#endif
 			cdj[j] = cdj[i];
+#if USE_SAT_SOLVER
 			cdj[i] = cdtmp; /* The SAT parser frees chosen_disjuncts elements. */
+#endif
 
 			remap[i] = j;
 			j++;
@@ -778,12 +785,14 @@ Linkage linkage_create(LinkageIdx k, Sentence sent, Parse_Options opts)
 {
 	Linkage linkage;
 
+#if USE_SAT_SOLVER
 	if (opts->use_sat_solver)
 	{
 		linkage = sat_create_linkage(k, sent, opts);
 		if (!linkage) return NULL;
 	}
 	else
+#endif
 	{
 		/* Cannot create a Linkage for a discarded linkage. */
 		if (sent->num_linkages_post_processed <= k) return NULL;
@@ -885,7 +894,7 @@ const char * linkage_get_disjunct_str(const Linkage linkage, WordIdx w)
 	return linkage->disjunct_list_str[w];
 }
 
-double linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
+float linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
 {
 	Disjunct *dj;
 
@@ -912,7 +921,7 @@ int linkage_unused_word_cost(const Linkage linkage)
 	return linkage->lifo.unused_word_cost;
 }
 
-double linkage_disjunct_cost(const Linkage linkage)
+float linkage_disjunct_cost(const Linkage linkage)
 {
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage) return 0.0;
